@@ -80,6 +80,11 @@
 //                    arithmetic. Import L_udmstar and L_utmslash from vm.s.
 //                    Now passes John Hayes' ANS core tests for <# # #S #>  km 
 //      06-19-2004  modified PrintVM_Error() to handle both kinds of errors km
+//      08-20-2004  modified CPP_include() to clear file stream error  cmb
+//      09-04-2004  modified OpenForth() and CPP_bye() to save and restore
+//                    initial terminal settings  km
+//      09-10-2004  modified CPP_refill() to advance line counter and added 
+//                    dump_return_stack() for debugging  km
 #include <string.h>
 #include <stdlib.h>
 #include <math.h>
@@ -131,6 +136,8 @@ extern "C" {
   // functions exported FROM vmc.c
 
   void set_start_time(void);
+  void save_term(void);
+  void restore_term(void);
 
   // vm functions exported FROM vm.s
 
@@ -166,7 +173,7 @@ extern "C" {
   char TIB[256];
   char NumberBuf[256];
 }
-
+extern "C" void dump_return_stack(void);
 extern "C" int JumpTable[];
 
 // The Dictionary
@@ -267,6 +274,7 @@ int OpenForth ()
     Base = 10;
     State = FALSE;
 
+    save_term();
     return Dictionary.size();
 }
 //---------------------------------------------------------------
@@ -1455,6 +1463,7 @@ int CPP_bye ()
 
   CloseForth();
   *pOutStream << "Goodbye.\n";
+  restore_term();
   exit(0);
 
   return 0;
@@ -1991,6 +2000,9 @@ int CPP_include()
     {
 	if(!strchr(filename,'/'))
 	{
+	    // Clear the previous error.
+	    f.clear();
+
 	    char temp[256];
 	    if (getenv("KFORTH_DIR"))
 	    { 
@@ -2055,8 +2067,10 @@ int CPP_source()
 
 int CPP_refill()
 {
-    pInStream->getline(TIB, 255);
-    *GlobalSp-- = (pInStream->fail()) ? FALSE : TRUE;
+    pInStream->getline(TIB, 255); 
+    int flag = (pInStream->fail()) ? FALSE : TRUE;
+    if (flag) ++linecount;
+    *GlobalSp-- = flag;
     *GlobalTp-- = OP_IVAL;
     pTIB = TIB;
     return 0;
@@ -2108,4 +2122,14 @@ int CPP_rpstore()
     return 0;
 }
 
+void dump_return_stack()  // for debugging purposes
+{
+    int* p = GlobalRp;
+    cout << endl << "Return Stack: " << endl;
+    while (p < BottomOfReturnStack)
+    {
+	++p; 
+	cout << (int)*p << endl; 
+    }
+}
 }
