@@ -3,7 +3,7 @@
 // FORTH compiler to generate FORTH Byte Code (FBC) from expressions
 //   or programs
 //
-// Copyright (c) 1998--2004 Krishna Myneni and David P. Wallace, 
+// Copyright (c) 1998--2005 Krishna Myneni and David P. Wallace, 
 //   Creative Consulting for Research and Education
 //
 // This software is provided under the terms of the General Public License.
@@ -92,6 +92,8 @@
 //      04-xx-2004 changed intrinsic wordlist specification to
 //                   structure array following suggestion of BK.
 //      06-19-2004 updated error handling method after call to ForthVM()
+//      09-05-2004 added FORTH-SIGNAL, RAISE, SET-ITIMER, and GET-ITIMER
+//      09-19-2005 added D>S
 #include <iostream>
 #include <fstream>
 using std::cout;
@@ -306,6 +308,10 @@ WordTemplate IntrinsicWords[] =
     { "EXECUTE",   OP_EXECUTE,      0 },
     { "CALL",      OP_CALL,         0 },
     { "SYSTEM",    OP_SYSTEM,       0 },
+    { "FORTH-SIGNAL", OP_FORTHSIGNAL, 0 },
+    { "RAISE",     OP_RAISE,        0 },
+    { "SET-ITIMER", OP_SETITIMER,   0 },
+    { "GET-ITIMER", OP_GETITIMER,   0 },
     { "TIME&DATE", OP_TIMEANDDATE,  0 },
     { "MS",        OP_MS,           0 },
     { "MS@",       OP_MSFETCH,      0 },
@@ -451,6 +457,7 @@ WordTemplate IntrinsicWords[] =
     { "DEG>RAD",   OP_DEGTORAD,     0 },
     { "RAD>DEG",   OP_RADTODEG,     0 },
     { "S>D",       OP_STOD,         0 },
+    { "D>S",       OP_DROP,         0 },
     { "S>F",       OP_STOF,         0 },
     { "D>F",       OP_DTOF,         0 },
     { "F>D",       OP_FTOD,         0 },
