@@ -78,6 +78,10 @@
 //                    arithmetic. Import L_udmstar and L_utmslash from vm.s.
 //                    Now passes John Hayes' ANS core tests for <# # #S #>  km 
 //      06-19-2004  modified PrintVM_Error() to handle both kinds of errors km
+//      08-20-2004  modified CPP_include() to clear file stream error  cmb
+//      09-04-2004  modified OpenForth() and CPP_bye() to save and restore
+//                    initial terminal settings  km
+//      09-10-2004  modified CPP_refill() to advance line counter  km
 #include <string.h>
 #include <stdlib.h>
 #include <math.h>
@@ -129,6 +133,8 @@ extern "C" {
   // functions exported FROM vmc.c
 
   void set_start_time(void);
+  void save_term(void);
+  void restore_term(void);
 
   // vm functions exported FROM vm.s
 
@@ -261,6 +267,7 @@ int OpenForth ()
     Base = 10;
     State = FALSE;
 
+    save_term();
     return Dictionary.size();
 }
 //---------------------------------------------------------------
@@ -1432,6 +1439,7 @@ int CPP_bye ()
 
   CloseForth();
   *pOutStream << "Goodbye.\n";
+  restore_term();
   exit(0);
 
   return 0;
@@ -1968,6 +1976,9 @@ int CPP_include()
     {
 	if(!strchr(filename,'/'))
 	{
+	    // Clear the previous error
+	    f.clear();
+
 	    char temp[256];
 	    if (getenv("KFORTH_DIR"))
 	    { 
@@ -2033,8 +2044,9 @@ int CPP_source()
 int CPP_refill()
 {
     pInStream->getline(TIB, 255);
-    *GlobalSp-- = (pInStream->fail()) ? FALSE : TRUE;
-
+    int flag = (pInStream->fail()) ? FALSE : TRUE;
+    if (flag) ++linecount;
+    *GlobalSp-- = flag;
     pTIB = TIB;
     return 0;
 }
