@@ -5,7 +5,7 @@
 //  Copyright (c) 1996--2004 Krishna Myneni, Creative Consulting for
 //    Research & Education
 //  Created: 2-24-96
-//  Last Revised: 2004-04-08
+//  Last Revised: 2004-09-10
 
 #ifndef __FORTHBYTECODES_H__
 #define __FORTHBYTECODES_H__
@@ -256,6 +256,11 @@
 #define OP_FPOW     186
 #define OP_FSQRT    187
 
+// SP! and RP!
+
+#define OP_SPSTORE  188
+#define OP_RPSTORE  189
+
 // floating pt relational
 
 #define OP_FEQ      190
@@ -362,8 +367,6 @@
 #define OP_STATE    264
 #define OP_ALLOCATE 265
 #define OP_FREE     266
-#define OP_SPSTORE  267
-#define OP_RPSTORE  268
 
 // 270 --
 
@@ -371,6 +374,11 @@
 #define OP_BRACKETCOMPILE 271
 #define OP_POSTPONE 272
 
+#define OP_FORTHSIGNAL 275
+#define OP_RAISE       276
+#define OP_SETITIMER   277
+#define OP_GETITIMER   278
+
 // 280--
 
 #define OP_TOFLOAT  280
@@ -382,5 +390,14 @@
 #define OP_FSINH    286
 #define OP_FTANH    287
 
+// 300--
+
+#define OP_DIV_FLOORED          300
+#define OP_MOD_FLOORED          301
+#define OP_SLASHMOD_FLOORED     302
+#define OP_STARSLASH_FLOORED    303
+#define OP_STARSLASHMOD_FLOORED 304
+#define OP_MSLASH_FLOORED       305
+#define OP_MSTARSLASH_FLOORED   306
 
 #endif
