@@ -2,7 +2,7 @@
 //
 // The kForth environment
 //
-// Copyright (c) 1998--2004 Krishna Myneni, 
+// Copyright (c) 1998--2005 Krishna Myneni, 
 //   Creative Consulting for Research and Education
 // 
 // This software is provided under the terms of the GNU General Public License.
@@ -24,8 +24,12 @@
 //
 //      kforth [name[.4th]] [-D] [-e string]
 //
-char* version = "1.2.6";
-char* Rls_Date = "2004-06-20";
+#ifdef VERSION
+char* version=VERSION;
+#else
+char* version="?";
+#endif
+char* build=BUILD_DATE;
 
 #include <iostream>
 #include <fstream>
@@ -55,6 +59,11 @@ extern "C" int* JumpTable;
 extern "C" int* BottomOfStack;
 extern "C" int* BottomOfReturnStack;
 extern "C" char TIB[];
+extern "C" {
+    void echo_on(void);
+    void echo_off(void);
+}
+
 int debug = 0;
 
 int main(int argc, char *argv[])
@@ -69,10 +78,9 @@ int main(int argc, char *argv[])
 
     if (argc < 2)
       {
-	cout << "kForth v " << version << "\t (Rls. " << Rls_Date << ")" << endl;
-	cout << "Copyright (c) 1998--2004 Krishna Myneni and David P. Wallace" 
-	     << endl;
-        cout << "Contributions by: gd mu bk abs tn cmb bg" << endl;
+	cout << "kForth v " << version << "\t (Build: " << build << ")" << endl;
+	cout << "Copyright (c) 1998--2005 Krishna Myneni" << endl;
+        cout << "Contributions by: dpw gd mu bk abs tn cmb bg" << endl;
 	cout << "Provided under the GNU General Public License." << endl << endl;
       }
 
@@ -137,8 +145,10 @@ int main(int argc, char *argv[])
 	       
             	pSS = new istringstream(s);
 	    }
-	    SetForthInputStream (*pSS);	    
+	    SetForthInputStream (*pSS);
+	    echo_off();
             ec = ForthCompiler (&op, &line_num);
+	    echo_on();
 	    delete pSS;
 	    pSS = NULL;
 
