@@ -24,6 +24,10 @@ Revisions:
                             C_ftanh
 	03-21-2004 -- recoded single argument math functions using a
                         MACRO based on BK's 3/20/04 revisions to vmc.c
+        09-04-2004 -- added functions save_term(), restore_term(),
+                        echo_on(), and echo_off() to fix echo problems
+                        with KEY?
+	09-05-2004 -- added signal handler interface
 */
 
 #include<sys/types.h>
@@ -32,6 +36,7 @@ Revisions:
 #include<sys/stat.h>
 #include<termios.h>
 #include<stdio.h>
+#include<signal.h>
 #include<unistd.h>
 #include<time.h>
 #include<fcntl.h>
@@ -44,7 +49,10 @@ Revisions:
 #define TRUE -1
 #define FALSE 0
 #define E_V_NOTADDR 1
+#define E_V_BADCODE 6
 #define E_V_STK_UNDERFLOW   7
+#define E_V_QUIT  8
+
 #define byte unsigned char
 
 extern int* GlobalSp;
@@ -58,12 +66,55 @@ extern byte* BottomOfTypeStack;
 extern byte* BottomOfReturnTypeStack;
 extern int Base;
 
+extern int vm(byte*);
+
 struct timeval ForthStartTime;
+struct termios tios0;
 double* pf;
 double f;
 char temp_str[256];
 char key_query_char = 0;
 
+// signal dispatch table
+
+void* signal_xtmap [32] =
+{
+    NULL,              //  1  SIGHUP, Hangup
+    NULL,              //  2  SIGINT, Interrupt
+    NULL,              //  3  SIGQUIT, Quit
+    NULL,              //  4  SIGILL, Illegal instruction
+    NULL,              //  5  SIGTRAP, Trace trap
+    NULL,              //  6  SIGABRT, Abort
+    NULL,              //  7  SIGBUS,  Bus error
+    NULL,              //  8  SIGFPE,  Floating-point exception
+    NULL,              //  9  SIGKILL, Kill (unblockable)
+    NULL,              // 10  SIGUSR1, User-defined
+    NULL,              // 11  SIGSEGV, Segmentation fault
+    NULL,              // 12  SIGUSR2, User-defined
+    NULL,              // 13  SIGPIPE, Broken pipe
+    NULL,              // 14  SIGALRM, Alarm clock
+    NULL,              // 15  SIGTERM, Termination
+    NULL,              // 16  SIGSTKFLT, Stack fault
+    NULL,              // 17  SIGCHLD, Child status changed
+    NULL,              // 18  SIGCONT, Continue execution
+    NULL,              // 19  SIGSTOP, Stop (unblockable)
+    NULL,              // 20  SIGTSTP, Keyboard stop
+    NULL,              // 21  SIGTTIN, Background read from tty
+    NULL,              // 22  SIGTTOU, Background write to tty
+    NULL,              // 23  SIGURG, Urgent condition on socket
+    NULL,              // 24  SIGXCPU, CPU time limit exceeded
+    NULL,              // 25  SIGXFSZ, File size limit exceeded
+    NULL,              // 26  SIGVTARM, Virtual alarm clock
+    NULL,              // 27  SIGPROF, Profiling alarm clock
+    NULL,              // 28  SIGWINCH, Window size change
+    NULL,              // 29  SIGPOLL,  Pollable event occured
+    NULL,              // 30  SIGPWR,  Power failure restart
+    NULL,              // 31  SIGUNUSED,  Not used
+    NULL
+};
+
+static void forth_signal_handler (int); 
+
 #define DOUBLE_FUNC(x)   pf = (double*)(GlobalSp+1); *pf=x(*pf);
   
 int C_ftan  () { DOUBLE_FUNC(tan)  return 0; }
@@ -222,6 +273,33 @@ int C_ioctl ()
 }
 /*----------------------------------------------------------*/
 
+void save_term ()
+{
+    tcgetattr(0, &tios0);
+}
+
+void restore_term ()
+{
+    tcsetattr(0, TCSANOW, &tios0);
+}
+
+void echo_off ()
+{
+  struct termios t;
+  tcgetattr(0, &t);
+  t.c_lflag &= ~ECHO;
+  tcsetattr(0, TCSANOW, &t);
+}
+
+void echo_on ()
+{
+  struct termios t;
+  tcgetattr(0, &t);
+  t.c_lflag |= ECHO;
+  tcsetattr(0, TCSANOW, &t);
+}
+/*----------------------------------------------------------*/
+
 int C_key ()
 {
   /* stack: ( -- n | wait for keypress and return key code ) */
@@ -263,7 +341,6 @@ int C_keyquery ()
 {
   /* stack: ( a -- b | return true if a key is available ) */
 
-  int result;
   char ch = 0;
   struct termios t1, t2;
 
@@ -282,7 +359,7 @@ int C_keyquery ()
       tcsetattr(0, TCSANOW, &t2);
 
       *GlobalSp-- = read(0, &ch, 1) ? -1 : 0;
-      if (ch) key_query_char = ch;
+      if (ch) key_query_char = ch;  
       tcsetattr(0, TCSANOW, &t1);
     }
 
@@ -593,4 +670,108 @@ int C_compare ()
 }
 /*------------------------------------------------------*/
 
+int C_setitimer ()
+{
+    // stack: ( timer-type avalue aoldvalue -- flag ) 
+    
 
+    int type, e;
+    struct itimerval *v1, *v2;
+
+    v2 = (struct itimerval*) *++GlobalSp; 
+    if (*++GlobalTp != OP_ADDR) return E_V_NOTADDR;
+    v1 = (struct itimerval*) *++GlobalSp;
+    if (*++GlobalTp != OP_ADDR) return E_V_NOTADDR;
+    type = *++GlobalSp; 
+    ++GlobalTp;
+
+    e = setitimer (type, v1, v2);
+
+    *GlobalSp-- = e; *GlobalTp-- = OP_IVAL;
+    return 0;
+}
+
+int C_getitimer ()
+{
+    // stack: ( timer-type  avalue -- flag )
+    
+    int type, e;
+    struct itimerval *v;
+
+    v = (struct itimerval*) *++GlobalSp;
+    if (*++GlobalTp != OP_ADDR) return E_V_NOTADDR;
+    type = *++GlobalSp; ++GlobalTp;
+
+    e = getitimer (type, v);
+
+    *GlobalSp-- = e; *GlobalTp-- = OP_IVAL;
+    return 0;
+}
+
+int C_raise ()
+{
+    // stack: ( signum -- ior )
+    int signum = *++GlobalSp;
+    *GlobalSp-- = raise(signum);
+    return 0;
+}
+
+int C_forth_signal ()
+{
+    // Install a Forth handler for specified signal 
+    // stack: ( xt n -- oldxt )
+
+    int signum;
+    void *xt, *oldxt;
+
+    signum = *++GlobalSp; ++GlobalTp;
+    if ((signum > 0) && (signum < 31))
+    {
+	++GlobalSp; ++GlobalTp;
+	oldxt = signal_xtmap[signum-1];
+	xt = (void *) *GlobalSp;
+	switch ((int) xt)
+	{
+	    case (int) SIG_DFL:
+		// Reset the default signal handler if xt = 0
+		signal (signum, SIG_DFL);
+		xt = 0;
+		break;
+	    case (int) SIG_IGN:
+		// Ignore the signal if xt = 1
+		signal (signum, SIG_IGN);
+		xt = 0;
+		break;
+	    default:
+		// All other xt s must be valid addresses to opcodes
+		if (*GlobalTp != OP_ADDR) return E_V_NOTADDR;
+		signal (signum, forth_signal_handler);
+		break;
+	}
+        signal_xtmap[signum-1] = xt;
+	*GlobalSp-- = (int) oldxt; *GlobalTp-- = OP_ADDR;
+    }
+    else
+	return E_V_BADCODE;
+
+    return 0;
+}
+/*-----------------------------------------------------*/
+
+static void forth_signal_handler (int signum)
+{
+    // Take the required action for the signal
+    //   by looking up and executing the appropriate
+    //   Forth word which has been designated to
+    //   handle this signal.
+
+    int e;
+    void* xt = signal_xtmap[signum-1];
+    *GlobalSp-- = signum; *GlobalTp-- = OP_IVAL;
+    if (xt)
+    {
+	e = vm((byte*) xt);
+	// printf ("\nvm returns %d", e);
+	// if (e == E_V_QUIT) we need to do a longjmp
+    }
+}
